import urllib.parse as urlparse
HOST = 'http://api.kassa.rambler.ru/v2/'

MOVIE_SCHEDULES_LINK = HOST + '{api_key}/json/movie/Schedule'
MOVIE_LIST_LINK = HOST + '{api_key}/json/Movie/list'
MOVIE_LINK = HOST + '{api_key}/json/Movie/Object'


def get_movie_schedules_link(**kwargs):
    api_key = kwargs.pop('api_key')
    return _set_params_to_url(MOVIE_SCHEDULES_LINK.format(api_key=api_key),
                              kwargs)


def get_movies_link(**kwargs):
    api_key = kwargs.pop('api_key')
    return _set_params_to_url(MOVIE_LIST_LINK.format(api_key=api_key), kwargs)


def get_movie_link(**kwargs):
    api_key = kwargs.pop('api_key')
    return _set_params_to_url(MOVIE_LINK.format(api_key=api_key), kwargs)


def get_widget_hallplan_url(**kwargs):
    return "https://widget.kassa.rambler.ru/place/hallplan" \
           "?sessionid={session_id}&WidgetID={widget_id}&GeoPlaceID={city_id}"\
        .format(**kwargs)


def _set_params_to_url(url, params):
    url_parts = list(urlparse.urlparse(url))
    query = dict(urlparse.parse_qsl(url_parts[4]))
    query.update(params)
    url_parts[4] = urlparse.urlencode(query)
    return urlparse.urlunparse(url_parts)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, validator


class Schedule(BaseModel):
    SessionID: int
    CityID: int
    CreationClassType: str
    CreationObjectID: int
    PlaceClassType: str
    PlaceObjectID: int
    DateTime: datetime
    Format: str
    IsSaleAvailable: bool
    IsReservationAvailable: bool
    IsWithoutSeats: bool
    MinPrice: Optional[int]
    MaxPrice: Optional[int]
    HallID: str
    HallName: str
    FeeType: str
    FeeValue: str

    @validator('MinPrice', 'MaxPrice', pre=True, whole=True)
    def str_to_int(cls, v):
        try:
            return int(v)
        except ValueError:
            return None



import requests
from rambler_kassa.exceptions import RamblerKassaException


class Request:
    @staticmethod
    def get(url):
        response = requests.get(url)
        try:
            data = response.json()
        except Exception as error:
            raise RamblerKassaException(error.args)
        if data.get('Code'):
            raise RamblerKassaException(data.get('Message'))
        return data



from enum import Enum
from typing import List

from rambler_kassa.models.movie import Movie
from rambler_kassa.models.schedule import Schedule
from rambler_kassa.request_manager import Request
from rambler_kassa.utils import *


class ScheduleType(Enum):
    Movie = 1


class RamblerKassa(object):

    def __init__(self, api_key: str, city_id: int, widget_id: int = 0):
        self.api_key = api_key
        self.city_id = city_id
        self.widget_id = widget_id

    def get_movies(self, **kwargs) -> List[Movie]:
        """Return of current films from the cinema."""
        data = Request \
            .get(get_movies_link(api_key=self.api_key,
                                 CityID=self.city_id, **kwargs))

        movies = []
        for item in data['List']:
            schedule = Movie(**item)
            movies.append(schedule)
        return movies

    def get_movie(self, movie_id):
        """Get detail info of movie by id"""
        data = Request \
            .get(get_movie_link(api_key=self.api_key, ObjectID=movie_id))
        return Movie(**data)

    def get_movie_schedules(self, movie_id: int, **kwargs):
        """Return schedules of movie by id."""
        return self._get_schedules(
            ScheduleType.Movie,
            ObjectID=movie_id,
            **kwargs,
        )

    def get_movies_schedules(self, **kwargs):
        """Return schedules of movies."""
        return self._get_schedules(
            ScheduleType.Movie,
            **kwargs,
        )

    def get_url_buy_ticket(self, session_id):
        if not self.widget_id:
            print("You do not set widget_id")
        return get_widget_hallplan_url(session_id=session_id,
                                       city_id=self.city_id,
                                       widget_id=self.widget_id)

    def _get_schedules(self, schedule_type: ScheduleType, **kwargs):
        url = None
        kwargs.update({
            'CityID': self.city_id,
        })
        if schedule_type.Movie:
            url = get_movie_schedules_link(api_key=self.api_key, **kwargs)

        data = Request.get(url)

        schedules = []
        for item in data['List']:
            schedule = Schedule(**item)
            schedules.append(schedule)

        return schedules



from datetime import date
from pydantic import BaseModel, validator
from typing import List, Optional


class Movie(BaseModel):
    OriginalName: str
    Genre: List[str]
    Country: List[str]
    ViewCountDaily: int
    AgeRestriction: str
    Thumbnail: str
    Cast: List[str]
    Description: str
    Director: List[str]
    CreatorName: Optional[str]
    CreatorObjectID: Optional[int]
    Year: str
    Duration: str
    HorizonalThumbnail: Optional[str]
    IsNonStop: bool
    SubCreationIDs: List[int]
    Rating: str
    Trailers: Optional[List[str]]
    Frames: Optional[List[str]]
    ReleaseDate: date
    KinoplanID: Optional[int]
    ObjectID: int
    ClassType: str
    Name: str
    AfishaClassID: int
    AfishaObjectID: int

    @validator('Genre', 'Country', 'Cast', 'Director', pre=True, whole=True)
    def str_to_list(cls, v):
        res = v.split(',')
        if not res or not res[0]:
            return []
        return [i.strip() for i in res]

    @validator('Trailers', pre=True, whole=True)
    def trailers(cls, v):
        return [i[2:] if '//' in i[0:2] else i for i in v]



import setuptools
from pathlib import Path

setuptools.setup(
    name="rambler-kassa",
    version="0.0.4",
    description=('API Rambler Kassa', ),
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    license="MIT",
    maintainer="NMelis",
    author='NMelis',
    url='https://github.com/NMelis/rambler-kassa',
    install_requires=[
        'requests==2.21.0',
        'bs4==0.0.1',
        'pydantic',
        'python-slugify==3.0.2'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)